// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"

namespace Dawn {
  class AssetManager;

  class AssetLoader {
    private:
      std::weak_ptr<AssetManager> assetManager;

    public:
      std::string typetest;
      const static std::string ASSET_TYPE;

      const std::string name;
      bool_t loaded = false;

      /**
       * Create an abstract Asset object.
       * 
       * @param name Name of the asset.
       */
      AssetLoader(
        const std::shared_ptr<AssetManager> assetManager,
        const std::string name
      );

      /**
       * Virtual function that will be called by the asset manager on a 
       * synchronous basis. This will only trigger if the blocks are false and
       * the loaded is also false.
       */
      virtual void updateSync() = 0;

      /**
       * Virtual function called by the asset manager asynchronously every tick.
       * This will only trigger if blocks are false and the loaded state is also
       * false.
       */
      virtual void updateAsync() = 0;

      /**
       * Returns the asset type.
       * 
       * @return The asset type.
       */
      virtual std::string getAssetType() const = 0;

      /**
       * Returns the asset manager.
       * 
       * @return The asset manager.
       */
      std::shared_ptr<AssetManager> getAssetManager();

      /**
       * Load the asset immediately, this is blocking on the main thread.
       */
      void loadImmediately();
      
      /**
       * Dispose the asset item.
       */
      virtual ~AssetLoader();
  };
}