// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "asset/AssetLoader.hpp"
#include "asset/AssetDataLoader.hpp"
#include "display/font/TrueTypeTexture.hpp"

namespace Dawn {
  enum class TrueTypeLoaderState {
    INITIAL,
    ASYNC_LOADING,
    ASYNC_DONE,
    SYNC_LOADING,
    SYNC_DONE
  };

  class TrueTypeLoader : public AssetLoader {
    protected:
      FT_Library fontLibrary;
      FT_Face face;
      AssetDataLoader loader;
      std::unordered_map<uint32_t, std::weak_ptr<TrueTypeTexture>> textures;
      enum TrueTypeLoaderState state = TrueTypeLoaderState::INITIAL;
      uint8_t *buffer = nullptr;

    public:
      const static std::string ASSET_TYPE;

      TrueTypeLoader(
        const std::shared_ptr<AssetManager> assetManager,
        const std::string name
      );

      void updateSync() override;
      void updateAsync() override;
      std::string getAssetType() const override;

      /**
       * Returns the texture for the given font size.
       * 
       * @param fontSize Font size to get the texture for.
       * @return Texture for the given character.
       */
      std::shared_ptr<TrueTypeTexture> getTexture(
        const uint32_t fontSize
      );

      ~TrueTypeLoader();
  };
}