// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneComponent.hpp"
#include "component/display/IRenderableComponent.hpp"

namespace Dawn {
  class Material :
    public SceneComponent,
    public IRenderableComponent
  {
    private:
      std::vector<std::shared_ptr<IShaderBase>> lockedShaders;

    protected:
      /**
       * Locks the shaders to be used for rendering.
       * 
       * @param shaderManager Shader manager to use.
       * @return List of shaders to be used.
       */
      virtual std::vector<std::shared_ptr<IShaderBase>> getLockedShaders(
        ShaderManager &shaderManager
      ) = 0;

    public:
      void onInit() override;
      void onDispose() override;
  };
}