// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SimpleTexturedMaterial.hpp"
#include "util/JSON.hpp"
#include "asset/loader/TextureLoader.hpp"

using namespace Dawn;

std::vector<
  std::shared_ptr<IShaderBase>
> SimpleTexturedMaterial::getLockedShaders(ShaderManager &shaderManager) {
  return {
    shaderManager.getShader<SimpleTexturedShader>()
  };
}

void SimpleTexturedMaterial::load(std::shared_ptr<SceneLoadContext> ctx) {
  if(ctx->data.contains("color")) {
    this->setColor(JSON::color(ctx->data["color"]));
  }

  if(ctx->data.contains("texture")) {
    auto asset = ctx->getAsset<TextureLoader>(
      ctx->data["texture"].get<std::string>()
    );
    this->setTexture(asset->getTexture());
  }
}

struct Color SimpleTexturedMaterial::getColor() {
  return this->data.data.color;
}

std::shared_ptr<Texture> SimpleTexturedMaterial::getTexture() {
  return this->texture;
}

void SimpleTexturedMaterial::setTexture(
  const std::shared_ptr<Texture> texture
) {
  this->texture = texture;
}

void SimpleTexturedMaterial::setColor(const struct Color color) {
  this->data.data.color = color;
}

std::vector<std::shared_ptr<IRenderPass>> SimpleTexturedMaterial::getPasses(
  struct RenderPassContext &ctx
) {
  this->data.data.model = this->getItem()->getWorldTransform();
  this->data.data.projection = ctx.camera->getProjection();
  this->data.data.view = ctx.camera->getItem()->getWorldTransform();
  auto textures = std::unordered_map<
    shadertexturebinding_t, std::shared_ptr<Texture>
  >();
  
  if(this->texture) {
    this->data.data.hasTexture = true;
    this->data.texture = 0;
    textures[this->data.texture] = this->texture;
  } else {
    this->data.data.hasTexture = false;
  }

  return {
    createRenderPass<SimpleTexturedShader, struct SimpleTexturedShaderData>(
      *this,
      data,
      textures
    )
  };
}
