// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItem.hpp"
#include "display/mesh/Mesh.hpp"

namespace Dawn {
  class MeshComponent : public SceneComponent {
    private:
      bool_t valid = false;

      /**
       * Buffers the mesh.
       */
      void buffer();

    protected:
      /**
       * Called when the mesh should be loaded.
       * 
       * @param ctx Context to load the mesh from.
       */
      virtual void meshLoad(std::shared_ptr<SceneLoadContext> ctx) = 0;

      /**
       * Called when the mesh should be buffered.
       * 
       * @param mesh Mesh to buffer.
       */
      virtual void meshBuffer(std::shared_ptr<Mesh> mesh) = 0;

      /**
       * Invalidates the mesh.
       */
      void invalidate();

    public:
      std::shared_ptr<Mesh> mesh;

      void onInit() override;
      void onDispose() override;
      void load(std::shared_ptr<SceneLoadContext> ctx) override;
  };
}