// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/mesh/Mesh.hpp"

namespace Dawn {
    class IRenderPass {
    public:
      std::shared_ptr<Mesh> mesh;

      /**
       * Binds the shader for this render pass.
       */
      virtual void bind() = 0;

      /**
       * Sets the data for this render pass to the shader.
       */
      virtual void setData() = 0;

      /**
       * Uploads the data to the GPU.
       */
      virtual void upload() = 0;

      /**
       * Draws the mesh for this render pass.
       */
      virtual void draw() = 0;

      /**
       * Cleans up the render pass.
       */
      virtual ~IRenderPass() {
      }
  };
}