// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawn.hpp"
#include "slang.h"

using namespace slang;

namespace Dawn {
  class ShaderEntry;

  class IShaderEntry {
    public:
      /**
       * Initialize the IShaderEntry object
       * 
       * @param stage The stage of the shader entry.
       * @param code The code of the shader entry.
       */
      virtual void init(
        const SlangStage &stage,
        const std::string &code
      ) = 0;

      /**
       * Destroy the IShaderEntry object
       */
      virtual ~IShaderEntry();
  };

  class IShaderProgram2 {
    protected:
      std::vector<std::shared_ptr<ShaderEntry>> entries;

    public:
      /**
       * Initialize the IShaderProgram2 object
       * 
       * @param entries The list of shader entries to initialize with.
       */
      virtual void init(
        const std::vector<std::shared_ptr<ShaderEntry>> &entries
      ) = 0;

      /**
       * Destroy the IShaderProgram2 object
       */
      virtual ~IShaderProgram2();
  };
}