// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "component/display/material/SimpleTexturedMaterial.hpp"

#define RPG_ENTITY_SIZE 32.0f

namespace Dawn {
  enum class RPGEntityDirection {
    NORTH,
    SOUTH,
    EAST,
    WEST
  };

  class RPGEntity final : public SceneComponent {
    private:

    protected:
      enum RPGEntityDirection facingDirection = RPGEntityDirection::SOUTH;

      void updateSprite();

    public:
      void onInit() override;
      void onDispose() override;

      /**
       * Returns the facing direction of this entity.
       * 
       * @return The facing direction of this entity.
       */
      enum RPGEntityDirection getFacingDirection();

      /**
       * Sets the facing direction of this entity.
       * 
       * @param dir The direction to face.
       */
      void setFacingDirection(const enum RPGEntityDirection dir);

  };
}